from ..linter import LinterRule, LinterFix
from ...repositories.requirements import RequirementsRepository


class AddMissingPackagesToRequirements(LinterFix):
    label = "Add missing packages to requirements.txt"
    description = "\n".join(
        [
            "Add the missing packages to the requirements.txt file.",
            "If the requirements.txt file does not exist, it will be created.",
        ]
    )

    def fix(self):
        requirements = RequirementsRepository.load()
        for package in RequirementsRepository.get_recommendation():
            requirements.add(package.name, package.version)
        RequirementsRepository.save(requirements)


class NoMissingPackagesInRequirements(LinterRule):
    label = "Missing packages in requirements.txt"
    type = "bug"
    description = "\n".join(
        [
            "The requirements.txt file should not contain packages that are being used in the project.",
        ]
    )
    fixes = [AddMissingPackagesToRequirements()]

    def is_valid(self) -> bool:
        return len(RequirementsRepository.get_recommendation()) == 0



from ..linter import LinterRule, LinterFix
from ...settings import Settings
from ...utils.file import files_from_directory


class AddEnvToAbstraIgnore(LinterFix):
    label = "Add env to abstra ignore"
    description = "\n".join(
        [
            "Add the .env file to the .abstraignore file.",
            "If the .abstraignore file does not exist, it will be created.",
        ]
    )

    def fix(self):
        abstraignore_file = Settings.root_path / ".abstraignore"
        with abstraignore_file.open("a") as file:
            file.write("\n.env")


class NoEnvInBundle(LinterRule):
    label = ".env in bundle"
    type = "security"
    description = "\n".join(
        ["The .env file should not be in the bundle.", "This is a security risk."]
    )
    fixes = [AddEnvToAbstraIgnore()]

    def is_valid(self) -> bool:
        env_file = Settings.root_path / ".env"

        bundle_files = files_from_directory(Settings.root_path)

        return env_file not in bundle_files



from ..linter import LinterRule
from ...utils.file import traverse_code
from ...repositories.json.classes import AbstraJSONRepository


class NoSyntaxErrors(LinterRule):
    label = "Syntax errors"
    type = "bug"
    fixes = []
    error = None

    def is_valid(self) -> bool:
        abstra_json = AbstraJSONRepository.load()
        for entrypoint in abstra_json.iter_entrypoints():
            try:
                for _ in traverse_code(entrypoint, raise_on_syntax_errors=True):
                    pass
            except SyntaxError as e:
                self.error = e
                self.error.filename = str(entrypoint)
                return False
        return True

    def make_description(self):
        return str(self.error)
